unit ModuleUtils;

interface

uses
	SysUtils, JwaWindows;

type

	{ TModuleEnumerator }

	TModuleEnumerator = class
	private
		FProcessId: Dword;
		FSnapModule: Handle;
		FCurrent, FNext: TModuleEntry32W;
		FHasNext: Boolean;
	public
		function GetCurrent: TModuleEntry32W;
		function MoveNext: Boolean;
		procedure Reset;
		property Current: TModuleEntry32W read GetCurrent;
	public
		constructor Create(AProcessId: Dword);
		destructor Destroy; override;
	end;

	{ TProcessModules }

	TProcessModules = object
	private
		FProcessId: Dword;
	public
		constructor Init(AProcessId: Dword);
		function GetEnumerator: TModuleEnumerator;
	end;

function Modules(ProcId: Dword): TProcessModules;
function GetModule(ProcId: dword; LibName: String; out Module: TModuleEntry32W): Boolean;
function GetModule(ProcId: dword; LibName: String): TModuleEntry32W;

implementation

function Modules(ProcId: Dword): TProcessModules;
begin
	Result.Init(ProcId);
end;

function GetModule(ProcId: dword; LibName: String; out Module: TModuleEntry32W
	): Boolean;
var
	LocalModule: TModuleEntry32W;
	LibNameUnicode: UnicodeString;
begin
	Result:= False;

	LibNameUnicode:= UnicodeString(LibName);

	for LocalModule in Modules(ProcId) do begin
		if stricomp(@LocalModule.szModule[0], PWideChar(LibNameUnicode)) = 0 then begin
			Module:= LocalModule;
			Result:= True;
			break;
		end;
	end;
end;

function GetModule(ProcId: dword; LibName: String): TModuleEntry32W;
begin
	if not GetModule(ProcId, LibName, Result) then
		raise Exception.CreateFmt('Module "%s" not found.', [LibName]);
end;

{ TProcessModules }

constructor TProcessModules.Init(AProcessId: Dword);
begin
	FProcessId:= AProcessId;
end;

function TProcessModules.GetEnumerator: TModuleEnumerator;
begin
	Result:= TModuleEnumerator.Create(FProcessId);
end;

{ TModuleEnumerator }

function TModuleEnumerator.GetCurrent: TModuleEntry32W;
begin
	Result:= FCurrent;
end;

function TModuleEnumerator.MoveNext: Boolean;
begin
	Result:= False;

	if FHasNext then begin
		FCurrent:= FNext;
		FHasNext:= Module32NextW(FSnapModule, FNext);
		Result:= True;
	end;
end;

procedure TModuleEnumerator.Reset;
begin
	ZeroMemory(@FNext, SizeOf(TModuleEntry32W));
	FNext.dwSize:= SizeOf(TModuleEntry32W);
	FHasNext:= Module32FirstW(FSnapModule, FNext);
end;

constructor TModuleEnumerator.Create(AProcessId: Dword);
var
	LastError: Dword;
begin
	FProcessId:= AProcessId;

	FSnapModule:= CreateToolhelp32Snapshot(
		TH32CS_SNAPMODULE + TH32CS_SNAPMODULE32,
		FProcessId
	);
	if FSnapModule = INVALID_HANDLE_VALUE then begin
		LastError:= GetLastError;

		raise Exception.CreateFmt(
			'Failed to access list of loaded modules.'#13#10 +
			'OS Error %d: "%s"', [
				LastError,
				SysErrorMessage(LastError)
			]
		);
	end;

	Reset;
end;

destructor TModuleEnumerator.Destroy;
begin
	CloseHandle(FSnapModule);
	inherited Destroy;
end;

end.

